package ru.amse.rakkate.crossword.logic;


import java.lang.IndexOutOfBoundsException;
import java.util.List;
import java.util.LinkedList;

public class Map implements IMap {
    
    private final Square myMatrix[][];
    private final int myHeight;
    private final int myWidth;
    private final List<IBlock> myBlocksByRow[];
    private final List<IBlock> myBlocksByCoulumn[];
    
    public Map(int a, int b) {
        myMatrix = new Square[a][b];
        myHeight = b;
        myWidth = a;
        myBlocksByRow = new List[myHeight];
        myBlocksByCoulumn = new List[myWidth];
        for (int i = 0; i < myWidth; i++) {
            for (int j = 0; j < myHeight; j++) {
                myMatrix[i][j] = Square.EMPTY;
            }
        }
        for (int i = 0; i < myHeight; i++) {
            myBlocksByRow[i] = new LinkedList<IBlock>();
        }
        for (int i = 0; i < myWidth; i++) {
            myBlocksByCoulumn[i] = new LinkedList<IBlock>();
        }
    }
    
    public int  getWidth() {
        return myWidth;
    }
    
    public int getHeight() {
        return myHeight;
    }
    
    public Square getCell(int x, int y) {
        if ((myWidth <= x) || (myHeight <= y ) || (x < 0) || (y < 0)) {
            throw new IndexOutOfBoundsException("IMap: getCell" + "x = " + x + "y = " + y + "Widhrt =" + myWidth + "Height = " + myHeight);
        } else {
            return myMatrix[x][y];
        }
    }
    
    private int getIndexEndBlock(int x, List<IBlock> list) {
        IBlock b = list.get(0);
        int end = b.getEnd();
        int i;
        for(i = 1; end < (x - 1); i++) {
            if(list.size() > i) {
                b = list.get(i);
                end = b.getEnd();
            } else {
                break;
            }
        }
        i--;
        return i;
    }
    
    private int getIndexStartBlock(int x, List<IBlock> list) {
        int i;
        IBlock b = list.get(0);
        int start = b.getStart();
        for(i = 1; start < (x + 1); i++) {
            if(list.size() > i) {
                b = list.get(i);
                start = b.getStart();
            } else {
                break;
            }
        }
        i--;
        return i;
    }
    
    public void setCell(int x, int y, Square z) {
        if ((myWidth <= x) || (myHeight <= y ) || (x < 0) || (y < 0)) {
            throw new IndexOutOfBoundsException("IMap: setCell" + "x = " + x + " y =" + y);
        } else {
            List<IBlock> listRow = myBlocksByRow[y];
            List<IBlock> listCoulumn = myBlocksByCoulumn[x];
            Square previousCell = myMatrix[x][y];
            myMatrix[x][y] = z;
            if (listRow.isEmpty()) {
                if (z == Square.BLACK) {
                    IBlock b = new Block(x, x);
                    listRow.add(b);
                }
            } else if ((( x - 1) >= 0) && ((myMatrix[x - 1][y]) == Square.BLACK)) {
                if (((x + 1) < myWidth) && (myMatrix[x + 1][y] == Square.BLACK)) {
                    int i = getIndexEndBlock(x, listRow);
                    IBlock b = listRow.get(i);
                    if (z == Square.BLACK) {
                        IBlock b2 = listRow.get(i + 1);
                        b.setEnd(b2.getEnd());
                        listRow.remove(i + 1); 
                    } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) { 
                        IBlock b2 = new Block(x + 1, b.getEnd());
                        b.setEnd(x - 1);
                        listRow.add(i + 1, b2);
                    }
                } else {
                    int i = getIndexEndBlock(x, listRow);
                    IBlock b = listRow.get(i);
                    if (z == Square.BLACK) {
                        b.setEnd((b.getEnd() + 1));
                    } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                        b.setEnd((b.getEnd() - 1));
                    }
                }
            } else if (((x + 1) < myWidth) && (myMatrix[x + 1][y] == Square.BLACK)) {
                int i = getIndexStartBlock(x, listRow);
                IBlock b = listRow.get(i);
                if (z == Square.BLACK) {
                    b.setStart(b.getStart() - 1);
                } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                    b.setStart(b.getStart() + 1);
                }
            } else {
                if (z == Square.BLACK) {
                    int i = getIndexStartBlock(x, listRow);
                    int j = getIndexEndBlock(x, listRow);
                    IBlock b1 = new Block(x, x);
                    IBlock b2 = listRow.get(i);
                    if (i == (j + 1)) {
                        listRow.add(j, b1);
                    } else if (b2.getStart() > x) {
                        listRow.add(i, b1);
                    } else {
                        listRow.add((i + 1), b1);
                    }
                } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                    int i = getIndexEndBlock(x + 1, listRow);
                    listRow.remove(i);
                }
            }
            if (listCoulumn.isEmpty()) {
                IBlock b = new Block(y, y);
                if (z == Square.BLACK) {
                    listCoulumn.add(b);
                }
            } else if (((y - 1) >= 0) && ((myMatrix[x][y - 1]) == Square.BLACK)) {
                if (((y + 1) < myHeight) && (myMatrix[x][y + 1] == Square.BLACK)) {
                    int i = getIndexEndBlock(y, listCoulumn);
                    IBlock b = listCoulumn.get(i);
                    if (z == Square.BLACK) {
                        IBlock b2 = listCoulumn.get(i + 1);
                        b.setEnd(b2.getEnd());
                        listCoulumn.remove(i + 1);
                    } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                        IBlock b2 = new Block(y + 1, b.getEnd());
                        b.setEnd(y - 1);
                        listCoulumn.add(i + 1, b2);    
                    }
                } else {
                    int i = getIndexEndBlock(y, listCoulumn);
                    IBlock b = listCoulumn.get(i);
                    if (z == Square.BLACK) {
                        b.setEnd((b.getEnd() + 1));
                    } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                        b.setEnd((b.getEnd() - 1));
                    }
                }
            } else if (((y + 1) < myHeight) && (myMatrix[x][y + 1] == Square.BLACK)) {
                int i = getIndexStartBlock(y, listCoulumn);
                IBlock b = listCoulumn.get(i);
                if (z == Square.BLACK) {
                    b.setStart(b.getStart() - 1);
                } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                    b.setStart((b.getStart() + 1));
                }
            } else {
                if (z == Square.BLACK) {
                    int i = getIndexStartBlock(y, listCoulumn);
                    int j = getIndexEndBlock(y, listCoulumn);
                    IBlock b1 = new Block(y, y);
                    IBlock b2 = listCoulumn.get(i);
                    if(i == (j + 1)) {
                        listCoulumn.add(j,b1);
                    } else if (b2.getStart() > x) {
                        listCoulumn.add(i, b1);
                    } else {
                        listCoulumn.add((i + 1), b1);
                    }
                } else if ((z == Square.EMPTY) && (previousCell == Square.BLACK)) {
                    int i = getIndexEndBlock(y + 1, listCoulumn);
                    listCoulumn.remove(i);    
                }
            }
        }
    }
    
    public List<IBlock> getBlocksByRow( int y) {
        return myBlocksByRow[y];
    }
    
    public List<IBlock> getBlocksByCoulumn(int x) {
        return myBlocksByCoulumn[x];
    }
    
    public int getSizeBlockRow(int i, int j) {
        int k = 1;
        Square z = Square.BLACK;
        for (int l = 1; z == Square.BLACK; l++) {
            if (((j - l) >= 0) && (myMatrix[i][j - l] == Square.BLACK)) {
                k++;
            } else {
                z = Square.EMPTY;
            }    
        }
        z = Square.BLACK;
        for (int l = 1; z == Square.BLACK; l++) {
            if (((j + l) < myHeight) && (myMatrix[i][j + l] == Square.BLACK)) {
                k++;
            } else {
                z = Square.EMPTY;
            }    
        }  
        return k;      
    }
    
    public int getSizeBlockCoulumn(int i, int j) {
       int k = 1;
       Square z = Square.BLACK;
       for (int l = 1; z == Square.BLACK; l++) {
            if (((i - l) >= 0) && (myMatrix[i - l][j] == Square.BLACK)) {
                k++;
            } else {
                z = Square.EMPTY;
            }    
        }
        z = Square.BLACK;
        for (int l = 1; z == Square.BLACK; l++) {
            if (((i + l) < myWidth) && (myMatrix[i + l][j] == Square.BLACK)) {
                k++;
            } else {
                z = Square.EMPTY;
            }    
        }  
        return k;           
    }
    
    public boolean controlMaximum(int i, int j, ITask task) {
        if (((( i - 1) >= 0) && (myMatrix[i - 1][j] == Square.BLACK))
        || ((((i + 1) < myWidth) && (myMatrix[i + 1][j] == Square.BLACK)))) {
            if ((((j - 1) >= 0) && (myMatrix[i][j - 1] == Square.BLACK))
            || ((((j + 1) < myHeight) && (myMatrix[i][j + 1] == Square.BLACK)))) {
                int k = getSizeBlockCoulumn(i, j);
                int m = task.getMaxBlock(task.getRowArray()[j]);
                int l = getSizeBlockRow(i, j);
                int n = task.getMaxBlock(task.getCoulumnArray()[i]);
                if ((k > m) || (l > n)) {
                    return false;
                } else {
                    return true;
                }
            } else {
                int k = getSizeBlockCoulumn(i, j);
                int m = task.getMaxBlock(task.getRowArray()[j]);
                if (k > m) {
                    return false;
                } else {
                    return true;
                }
            }    
        } else {
            if ((((j - 1) >= 0) && (myMatrix[i][j - 1] == Square.BLACK))
            || ((((j + 1) < myHeight) && (myMatrix[i][j + 1] == Square.BLACK)))) {
                int k = getSizeBlockRow(i, j);
                int m = task.getMaxBlock(task.getCoulumnArray()[i]);
                if (k > m) {
                    return false;
                } else {
                    return true;
                }  
            } else {
                return true;
            }
        }
    }
    
    public boolean controlQuantity(int i, int j, ITask task) {
        if (((( i - 1) >= 0) && (myMatrix[i - 1][j] == Square.BLACK))
        || ((((i + 1) < myWidth) && (myMatrix[i + 1][j] == Square.BLACK)))) {
            if ((((j - 1) >= 0) && (myMatrix[i][j - 1] == Square.BLACK))
            || ((((j + 1) < myHeight) && (myMatrix[i][j + 1] == Square.BLACK)))) {
                return true;
            } else {
                if (task.getQuantityBlocksCoulumn(i) <= getBlocksByCoulumn(i).size()) {
                    return false;
                } else {
                    return true;
                }  
            }              
        } else {
            if (task.getQuantityBlocksRow(j) > getBlocksByRow(j).size()) {
                if ((((j - 1) >= 0) && (myMatrix[i][j - 1] == Square.BLACK))
                || ((((j + 1) < myHeight) && (myMatrix[i][j + 1] == Square.BLACK)))) {
                    return true;
                } else {
                    if (task.getQuantityBlocksCoulumn(i) <= getBlocksByCoulumn(i).size()) {
                        return false;
                    } else {
                        return true;
                    }
                }
            } else {
                return false;
            }
        }
    }
    
    public List<Integer>[] getArrayRow() {
       List<Integer> array[] = new List[myHeight];
       for (int i = 0; i < myHeight; i++) {
           array[i] = new LinkedList<Integer>();
           for (IBlock b: myBlocksByRow[i]) {
               array[i].add(b.getEnd() - b.getStart() + 1);
           }
       }
       return array;
    }
    
    public List<Integer>[] getArrayCoulumn() {
        List<Integer> array[] = new List[myWidth];
        for (int i = 0; i < myWidth; i++) {
            array[i] = new LinkedList<Integer>();
            for (IBlock b: myBlocksByCoulumn[i]) {
                array[i].add(b.getEnd() - b.getStart() + 1);
            }
        }
        return array;
    }
    
}
