package ru.amse.rakkate.crossword.ui.editor;

import ru.amse.rakkate.crossword.logic.*;
import ru.amse.rakkate.crossword.logic.loadsave.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import javax.xml.*;
import org.xml.sax.*;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.ParserConfigurationException;
import ru.amse.rakkate.crossword.ui.*;


public class EditorFrame {
     private JFrame myFrame;
     private final int mySize = 25; 
     private EditorView myEditorView;
     private IMap myMap;
     private ITask myTask;
     private MyJDialog myDialog;
     private String myTaskFile = null;
     private String myMapFile = null;
     
     public EditorFrame() {
        myFrame = new JFrame("EDITOR");
        myFrame.addWindowListener(new MyWindowListener());
     }
     
     public void showMenu() {
         myFrame.setSize(300, 100);
         myMap = new Map(0, 0);
         myTask = new Task(0, 0);
         JMenuBar menu = new JMenuBar();
         JToolBar toolBar = new JToolBar();
         myFrame.setJMenuBar(menu);
         JMenu file = new JMenu("File");
         menu.add(file);
         NewAction task = new NewAction("New");
         toolBar.add(task);
         file.add(task);
         OpenMapAction openMap = new OpenMapAction("Open map");
         file.add(openMap);
         toolBar.add(openMap);
         SaveMapAsAction saveMapAs = new SaveMapAsAction("Save map as");
         file.add(saveMapAs);
         SaveMapAction saveMap = new SaveMapAction("Save Map");
         file.add(saveMap);
         toolBar.add(saveMap);
         SaveTaskAsAction saveTaskAs = new SaveTaskAsAction("Save task as");
         file.add(saveTaskAs);
         SaveTaskAction saveTask = new SaveTaskAction("Save task");
         file.add(saveTask);
         toolBar.add(saveTask);
         file.addSeparator();
         file.add(new ExitAction("Exit"));   
         myEditorView = new EditorView(myMap, myTask, mySize); 
         myFrame.getContentPane().add(toolBar, BorderLayout.NORTH);
         myFrame.getContentPane().add(new JScrollPane(myEditorView));
         myFrame.setDefaultCloseOperation(myFrame.EXIT_ON_CLOSE);
         myFrame.setVisible(true);
     }
     
     public void saveTaskAs() {
         JFileChooser chooser = new JFileChooser();
         chooser.setDialogTitle("Will choose a file for preservation of a crossword puzzle");
         chooser.setApproveButtonText("Save");
         int res = chooser.showOpenDialog(myFrame);
         if (res == JFileChooser.APPROVE_OPTION) {
             try {
                 File p = chooser.getSelectedFile();
                 String fileName = p.getAbsolutePath();
                 if (!p.getName().toLowerCase().endsWith(".xml")) {
                     fileName = p.getAbsolutePath() + ".xml";
                 }
                 myTaskFile = fileName;
                 FileWriter v = null;
                 try {
                     v = new FileWriter(fileName);
                     XMLWriterTask writer = new XMLWriterTask();
                     writer.writeXML(v, myMap.getWidth(), myMap.getHeight(), myMap.getArrayRow(), myMap.getArrayCoulumn());
                     myEditorView.setUnchanged();
                 } finally {
                     if (v != null) {
                         v.close();
                     }
                 }  
             } catch (IOException e) {
                 JOptionPane.showMessageDialog(myFrame, e.getMessage());
             }
         }
     }
     
     public void saveTask() {
         if (myTaskFile == null) {
             saveTaskAs();
         } else {
             try {
                 FileWriter v = null;
                 try {
                     v = new FileWriter(myMapFile);
                     XMLWriterMap writer = new XMLWriterMap();
                     writer.writeMap(v, myMap, null, false);
                     myEditorView.setUnchanged();
                 } finally {
                     if (v != null) {
                         v.close();
                     }
                 }  
                 myEditorView.setUnchanged();   
             } catch (IOException e) {
                 JOptionPane.showMessageDialog(myFrame, e.getMessage());
             }
         }
     }  
      
     public int saveMap() {
         int res = 0;
         if (myMapFile == null) {
             res = saveMapAs();
         } else {
             try {
                 FileWriter v = null;
                 try {
                     v = new FileWriter(myMapFile);
                     XMLWriterMap writer = new XMLWriterMap();
                     writer.writeMap(v, myMap, null, false);
                     myEditorView.setUnchanged();
                 } finally {
                     if (v != null) {
                         v.close();
                     }
                 }  
             } catch (IOException e) {
                 JOptionPane.showMessageDialog(myFrame, e.getMessage());
             }
         }
         return res;
     }   
      
     public int saveMapAs() {
         JFileChooser chooser = new JFileChooser();
         chooser.setDialogTitle("Will choose a file for preservation of a crossword puzzle");
          chooser.setApproveButtonText("Save");
         int res = chooser.showOpenDialog(myFrame);
         if (res == JFileChooser.APPROVE_OPTION) {
             try {
                 File p = chooser.getSelectedFile();
                 String fileName = p.getAbsolutePath();
                 if (!p.getName().toLowerCase().endsWith(".xml")) {
                     fileName = p.getAbsolutePath() + ".xml";
                 }
                 myMapFile = fileName;
                 FileWriter v = null;
                 try {
                     v = new FileWriter(fileName);
                     XMLWriterMap writer = new XMLWriterMap();
                     writer.writeMap(v, myMap, null, false);
                 } finally {
                     if (v != null) {
                         v.close();
                     }
                 }   
                 myEditorView.setUnchanged();
             } catch (FileNotFoundException e) {
                JOptionPane.showMessageDialog(myFrame, e.getMessage());    
             } catch (IOException e) {
                 JOptionPane.showMessageDialog(myFrame, e.getMessage());
             }
         }
         return res;
     }
     
      
     private ImageIcon createIcon(String name) {
         URL url = EditorFrame.class.getResource(name);
	 return new ImageIcon(url);
     }
     private boolean saveIfNeeded() {
         if (myEditorView.getIsChanged()) {
             int res = JOptionPane.showConfirmDialog(myFrame, "Save?");
             switch(res) {
                 case JOptionPane.YES_OPTION:
                     int k  = saveMap();
                     if (k == JFileChooser.APPROVE_OPTION) {
                         return true;
                     } else {
                         return false;
                     }
                 case JOptionPane.NO_OPTION:
                     return true;
                 case JOptionPane.CANCEL_OPTION:
                     return false;        
             }
         }
         return true;
     }       
     
     private class NewAction extends AbstractAction {
        
         public NewAction(String name) {
             putValue(NAME, name);
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("F2"));
             putValue(AbstractAction.SMALL_ICON, createIcon("icon/2.gif"));
             putValue(AbstractAction.SHORT_DESCRIPTION, "New...");
         }
        
         public void actionPerformed(ActionEvent ae) {
             if (!saveIfNeeded()) {
                 return;
             }
            
             myDialog = new MyJDialog(myFrame, "Map");
             myDialog.run();
             myMap = myDialog.getMap();
             myTask = new Task(0, 0);
             myTaskFile = null;
             myEditorView.setTaskandMap(myTask, myMap, mySize);
             myFrame.setSize(((myMap.getWidth() + 4) * (mySize)), ((myMap.getHeight() + 4) * mySize));  
         }
     }
         
     private class ExitAction extends AbstractAction {
        
         public ExitAction(String name){
             putValue(NAME, name);
         }
        
         public void actionPerformed(ActionEvent ae) {
             if (!saveIfNeeded()) {
                 return;
             }
             System.exit(0);
         }
     }
    
     private class SaveMapAction extends AbstractAction {
        
         public SaveMapAction(String name) {
             putValue(NAME, name);
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl Z"));
             putValue(AbstractAction.SMALL_ICON, createIcon("icon/8.gif"));
             putValue(AbstractAction.SHORT_DESCRIPTION, "Save Map ...");
         } 
        
         public void actionPerformed(ActionEvent ae) {
            saveMap();
         }
     }
    
     private class OpenMapAction extends  AbstractAction {
        
         public OpenMapAction(String name) {
             putValue(NAME, name);
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl O"));
             putValue(AbstractAction.SMALL_ICON, createIcon("icon/3.gif"));
             putValue(AbstractAction.SHORT_DESCRIPTION, "Open Map...");
         }
        
         public void actionPerformed(ActionEvent ae) {
             if (!saveIfNeeded()) {
                 return;
             }
             JFileChooser chooser = new JFileChooser();
             chooser.setDialogTitle("choose a file");
             int res = chooser.showOpenDialog(myFrame);
             chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
             if (res == JFileChooser.APPROVE_OPTION) {
                 String nameFile = chooser.getSelectedFile().getAbsolutePath();    
                 myMapFile = nameFile;
                 try {
                     InputStream rm = null;
                     try {
                         MapReader mr = new MapReader();
                         rm = new FileInputStream(nameFile);
                         myMap = mr.readerMap(rm);
                     } finally {
                         if (rm != null) {
                             rm.close();
                         }
                     }
                     myEditorView.setTaskandMap(myTask, myMap, mySize);
                     myEditorView.setUnchanged();
                     myFrame.setSize(((myMap.getWidth() + myTask.getMaxRow() + 2) * (mySize)),((myMap.getHeight() + 4 + myTask.getMaxCoulumn()) * mySize));      
                 } catch (ParserConfigurationException e) {
                     JOptionPane.showMessageDialog(myFrame, e.getMessage());    
                 } catch (SAXException e) {
                     JOptionPane.showMessageDialog(myFrame, e.getMessage());    
                 } catch (IOException e) {
                     JOptionPane.showMessageDialog(myFrame, e.getMessage());
                 }
             }
         }   
     }
    
     private class SaveTaskAction extends AbstractAction {
         
         public SaveTaskAction(String name) {
             putValue(NAME, name);
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl D"));
             putValue(AbstractAction.SMALL_ICON, createIcon("icon/7.gif"));
             putValue(AbstractAction.SHORT_DESCRIPTION, "Save Task...");
         }
         
         public void actionPerformed(ActionEvent ae) {
             saveTask();
         }
     } 
    
     private class SaveMapAsAction extends AbstractAction {
         
         public SaveMapAsAction(String name) {
             putValue(NAME, name);
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("shift ctrl S"));
         }
         
         public void actionPerformed(ActionEvent ae) {
             saveMapAs();
         }
     }
    
     private class SaveTaskAsAction extends AbstractAction {
        
         public SaveTaskAsAction(String name) {
             putValue(NAME, name);
             putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("shift S"));
         }
        
         public void actionPerformed(ActionEvent ae) {
             saveTaskAs();                
         }
     }
     
     private class MyWindowListener extends WindowAdapter {
        
       public void windowClosing(WindowEvent we) {
           if (!saveIfNeeded()) {
              return;
           }  
       } 
   }
    
}
