package ru.amse.rakkate.crossword.ui.main;

import ru.amse.rakkate.crossword.logic.*;
import ru.amse.rakkate.crossword.logic.loadsave.*;
import java.util.List;
import java.util.LinkedList;
import javax.xml.*;
import org.xml.sax.*;
import java.net.*;

import java.awt.image.BufferedImage;
import java.util.Stack;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.ParserConfigurationException;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import javax.swing.filechooser.*;
import ru.amse.rakkate.crossword.ui.*;

public class GaimFrame {
    private JFrame myFrame;
    private IMap myMap;
    private ITask myTask;
    private MapView myMapView;
    private final int mySize = 25;
    private String myNameTaskFile;
    private String myNameSolutionFile = null;
    
    public GaimFrame() {
        myFrame = new JFrame("Crossword");
        myFrame.addWindowListener(new MyWindowListener());
    }
    
    public void showMenu() {
        myMap = new Map(0, 0);
        myTask = new Task(0, 0);
        myMapView = new MapView(myMap, myTask, mySize);
        myFrame.setSize(300, 100);
        JMenuBar menu = new JMenuBar();
        JToolBar toolBar = new JToolBar();
        myFrame.setJMenuBar(menu);
        JMenu file = new JMenu("File");
        JMenu result = new JMenu("Check Solution");
        menu.add(file);
        menu.add(result);
        ResultAction res = new ResultAction("CheckSolution");
        result.add(res);
        NewTaskAction task = new NewTaskAction("Open Task");
        toolBar.add(task);
        file.add(task);
        OpenSolutionAction openSolution = new OpenSolutionAction("Open Solution");
        file.add(openSolution);
        toolBar.add(openSolution);
        SaveActionAs saveAs = new SaveActionAs("Save Solution As");
        file.add(saveAs);
        SaveAction save = new SaveAction("Save Solution");
        file.add(save);
        toolBar.add(save);
        file.addSeparator();
        file.add(new ExitAction("Exit"));
        toolBar.add(res);
        myFrame.getContentPane().add(toolBar, BorderLayout.NORTH);
        myFrame.getContentPane().add(new JScrollPane(myMapView));
        myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        myFrame.setVisible(true);
    }
     
    private ImageIcon createIcon(String name) {
        URL url = GaimFrame.class.getResource(name);
	return new ImageIcon(url);
    }
    
    
    private boolean saveIfNeeded() {
        if (myMapView.getIsChanged()) {
            int res = JOptionPane.showConfirmDialog(myFrame, "Save?");
            switch(res) {
                case JOptionPane.YES_OPTION:
                    int k = saveSolution();
                    if (k == JFileChooser.APPROVE_OPTION) {
                        return true;
                    } else {
                        return false;
                    }
                case JOptionPane.NO_OPTION:
                    return true;
                case JOptionPane.CANCEL_OPTION:
                    return false;        
            }
        }
        return true;
    }        
    
    public int saveSolution() {
        int res = 0;
        if (myNameSolutionFile == null) {
            res = saveSolutionAs();
        } else {
            try {
                FileWriter v = null;
                try {
                    v = new FileWriter(myNameSolutionFile);
                    XMLWriterMap writer = new XMLWriterMap();
                    writer.writeMap(v, myMap, myNameTaskFile, true);
                } finally {
                    if (v != null) {
                        v.close();
                    }
                }  
                myMapView.setUnchanged();   
            } catch (IOException e) {
               JOptionPane.showMessageDialog(myFrame, e.getMessage());
            }
        }
        return res;
    }
    
    public int saveSolutionAs() {
        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Will choose a file for preservation of a crossword puzzle");
        chooser.setApproveButtonText("Save");
        int res = chooser.showOpenDialog(myFrame);
        if (res == JFileChooser.APPROVE_OPTION) {
            try {
                File p = chooser.getSelectedFile();
                String fileName = p.getAbsolutePath();
                if (!p.getName().toLowerCase().endsWith(".xml")) {
                    fileName = p.getAbsolutePath() + ".xml";
                }
                myNameSolutionFile = fileName;
                FileWriter v = null;
                try {
                    v = new FileWriter(fileName);
                    XMLWriterMap writer = new XMLWriterMap();
                    writer.writeMap(v, myMap, myNameTaskFile, true);
                } finally {
                    if (v != null) {
                        v.close();
                    }
                }  
                myMapView.setUnchanged();
            } catch (FileNotFoundException e) {
               JOptionPane.showMessageDialog(myFrame, e.getMessage());    
            } catch (IOException e) {
                JOptionPane.showMessageDialog(myFrame, e.getMessage());
            }
        }
        return res;
    }
    
    private class NewTaskAction extends AbstractAction {
        
        public NewTaskAction(String name) {
            putValue(NAME, name);
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("F2"));
            putValue(AbstractAction.SMALL_ICON, createIcon("icon/2.gif"));
            putValue(AbstractAction.SHORT_DESCRIPTION, "New Crossword...");
        }
        
        public void actionPerformed(ActionEvent ae) {
            if (!saveIfNeeded()) {
                return;
            }
            myNameSolutionFile = null;
            JFileChooser chooser = new JFileChooser();
            chooser.setDialogTitle("Will choose a xml file for loading a crossword puzzle");
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            int res = chooser.showOpenDialog(myFrame);
            if (res == JFileChooser.APPROVE_OPTION) {
                myNameTaskFile = chooser.getSelectedFile().getAbsolutePath();
                TaskReader r = new TaskReader();
                try {
                    InputStream is = null;
                    try {
                        is = new FileInputStream(myNameTaskFile);
                        myTask = r.reader(is); 
                    } finally {
                        if (is != null) {
                            is.close();
                        }
                    }
                    myMap = new Map(myTask.getWidth(), myTask.getHeight());
                    myMapView.setTaskandMap(myTask, myMap, mySize);
                    myFrame.setSize(((myMap.getWidth() + myTask.getMaxRow() + 2) * (mySize)), ((myMap.getHeight() + 4 + myTask.getMaxCoulumn()) * mySize));
                } catch (FileNotFoundException e) {
                    JOptionPane.showMessageDialog(myFrame, e.getMessage());
                } catch (IOException e) {
                    JOptionPane.showMessageDialog(myFrame, e.getMessage());    
                } catch (SAXException e) {
                    JOptionPane.showMessageDialog(myFrame, e.getMessage());
                } catch (ParserConfigurationException e) {
                    JOptionPane.showMessageDialog(myFrame,e.getMessage());
                }
             }
        }
    }
    
    private class ExitAction extends AbstractAction {
        
        public ExitAction(String name){
            putValue(NAME, name);
        }
        
        public void actionPerformed(ActionEvent ae) {
            if (!saveIfNeeded()) {
                return;
            }
            System.exit(0);
 
        }
    }
   
    private class OpenSolutionAction extends  AbstractAction {
        
        public OpenSolutionAction(String name) {
            putValue(NAME, name);
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl O"));
            putValue(AbstractAction.SMALL_ICON, createIcon("icon/3.gif"));
            putValue(AbstractAction.SHORT_DESCRIPTION, "Open Solution...");
        }
        
        public void actionPerformed(ActionEvent ae) {
            if (!saveIfNeeded()) {
                return;
            }
            JFileChooser chooser = new JFileChooser();
            chooser.setDialogTitle("Will choose a file for loading the kept crossword puzzle");
            int res = chooser.showOpenDialog(myFrame);
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            if (res == JFileChooser.APPROVE_OPTION) {
                String nameFile = chooser.getSelectedFile().getAbsolutePath();    
                myNameSolutionFile = nameFile;
                try {
                    InputStream r = null;
                    InputStream rm = null;
                    InputStream taskRead = null;
                    try {
                        r = new FileInputStream(nameFile);
                        TaskReader reader = new TaskReader();
                        MapReader mr = new MapReader();
                        myNameTaskFile = reader.readerNameTask(r);
                        taskRead = new FileInputStream(myNameTaskFile);
                        myTask = reader.reader(taskRead);
                        rm = new FileInputStream(nameFile);
                        myMap = mr.readerMap(rm);
                    } finally {
                        if (r != null) {
                            r.close();
                        }
                        if (rm != null) {
                            rm.close();
                        }
                        if (taskRead != null) {
                            taskRead.close();
                        }
                    }
                    myMapView.setTaskandMap(myTask, myMap, mySize);
                    myMapView.setUnchanged();
                    myFrame.setSize(((myMap.getWidth() + myTask.getMaxRow() + 2) * (mySize)),((myMap.getHeight() + 4 + myTask.getMaxCoulumn()) * mySize));      
                } catch (ParserConfigurationException e) {
                    JOptionPane.showMessageDialog(myFrame, e.getMessage());    
                } catch (SAXException e) {
                    JOptionPane.showMessageDialog(myFrame, e.getMessage());    
                } catch (IOException e) {
                    JOptionPane.showMessageDialog(myFrame, e.getMessage());
                }
            }
        }   
    }

    private class SaveAction extends AbstractAction {
        
        public SaveAction(String name) {
            putValue(NAME, name);
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl Z"));
            putValue(AbstractAction.SMALL_ICON, createIcon("icon/8.gif"));
            putValue(AbstractAction.SHORT_DESCRIPTION, "Save...");
        }
        
        public void actionPerformed(ActionEvent ae) {
            if ((myMap == null) && (myTask == null)) {
                JOptionPane.showMessageDialog(myFrame, "The crossword puzzle is not loaded");
            } else {
               saveSolution();
            }
        }
    }
    
    private class SaveActionAs extends AbstractAction {
        
        public SaveActionAs(String name) {
            putValue(NAME, name);
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("shift S"));
        }
        
        public void actionPerformed(ActionEvent ae) {
            if ((myMap == null) && (myTask == null)) {
                JOptionPane.showMessageDialog(myFrame, "The crossword puzzle is not loaded");
            } else {
                saveSolutionAs();          
            }
        }
    }
    
    private class ResultAction  extends AbstractAction {
        
        public ResultAction(String name) {
            putValue(NAME, name);
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("F5"));
            putValue(AbstractAction.SMALL_ICON, createIcon("icon/1.gif"));
            putValue(AbstractAction.SHORT_DESCRIPTION, "Result");
        }
        
        public void actionPerformed(ActionEvent ae) {
            if ((myMap != null) && (myTask != null)) {
                boolean result = myTask.getResult(myMap);
                if (result) {
                     JOptionPane.showMessageDialog(myFrame, "You have won!");
                } else {
                     JOptionPane.showMessageDialog(myFrame, "The answer incorrect");
                }
            } else {
                JOptionPane.showMessageDialog(myFrame, "The crossword puzzle is not loaded");
            }
        }
    }
    
    private class MyWindowListener extends WindowAdapter {
        
        public void windowClosing(WindowEvent we) {
            if (!saveIfNeeded()) {
               return;
            }  
        }
    }
    
}
